unit fRSAtest;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, RSA, ExtCtrls, SHA;

type
  TfrmRSATest = class(TForm)
    RSA1:          TRSA;
    btnCreateKey:  TButton;
    btnEncrypt:    TButton;
    edtPublicKey:  TEdit;
    edtPrivateKey: TEdit;
    sbStatus:      TStatusBar;
    edtTestText:   TEdit;
    btnDecrypt:    TButton;
    grpKeyLength:  TGroupBox;
    rbn512:        TRadioButton;
    rbn1024:       TRadioButton;
    rbn2048:       TRadioButton;
    lblPrivateKey: TLabel;
    lblPublicKey:  TLabel;
    lblTestText:   TLabel;
    btnPublicKeySet: TButton;
    btnPrivateKeySet: TButton;
    edtSignature: TEdit;
    lblSignature: TLabel;
    btnSign: TButton;
    btnVerify: TButton;
    btnSignatureSet: TButton;
    btnSymKey: TButton;
    bvlVersion: TBevel;
    lblVersion: TLabel;
    edtVersion: TEdit;
    btnPubKeyLoad: TButton;
    btnPrivKeyLoad: TButton;
    SHA1: TSHA;
    procedure      btnCreateKeyClick(Sender: TObject);
    procedure      RSA1StatusUpdate(Sender: TObject);
    procedure      rbn512Click(Sender: TObject);
    procedure      rbn1024Click(Sender: TObject);
    procedure      rbn2048Click(Sender: TObject);
    procedure      btnEncryptClick(Sender: TObject);
    procedure      btnDecryptClick(Sender: TObject);
    procedure btnPublicKeySetClick(Sender: TObject);
    procedure btnPrivateKeySetClick(Sender: TObject);
    procedure btnSignClick(Sender: TObject);
    procedure btnVerifyClick(Sender: TObject);
    procedure btnSignatureSetClick(Sender: TObject);
    procedure btnSymKeyClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnPubKeyLoadClick(Sender: TObject);
    procedure btnPrivKeyLoadClick(Sender: TObject);
  end;

var
  frmRSATest: TfrmRSATest;

implementation

{$R *.DFM}

{*******************************************************************************
* Procedure : TfrmRSATest.btnCreateKeyClick                                    *
********************************************************************************
* Purpose   : Calls the TRSA procedure for making key pairs using the currently*
*             selected key length. The seed is used as the basis for creating  *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.btnCreateKeyClick(Sender: TObject);
var
     SeedP: BigNum;         // holds the seed for the random search of Prime1
     SeedQ: BigNum;         // holds the seed for the random search of Prime2
     PrimeLength: Integer;  // holds the number of significant 32 bit words
     i: integer;            // loop counter
begin
     {** get a random seed for the search - your routine
         should use a high quality RNG to fill this seed **}

     // get the number of words in the BigNum for the currently
     // selected key length
     PrimeLength := RSA1.GetPrecision;

     // fill the random numbers which will seed the search engine
     for i := 0 to PrimeLength do
     begin
          SeedP.Contents[i] := Random($FFFFFFFF);
          SeedQ.Contents[i] := Random($FFFFFFFF);
     end;

     // create the key pair
     RSA1.MakeKeyPair(SeedP, SeedQ);

     // show the user the values
     edtPublicKey.Text := RSA1.PublicKey;
     edtPrivateKey.Text := RSA1.PrivateKey;
end; {TfrmRSATest.btnCreateKeyClick}

{*******************************************************************************
* Procedure : TfrmRSATest.btnEncryptClick                                      *
********************************************************************************
* Purpose   : Calls the TRSA procedure for encrypting data using the currently *
*             loaded keys                                                      *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.btnEncryptClick(Sender: TObject);
var
     TempString: string;
begin
     // encrypt the string in the edit box
     RSA1.EncryptString(edtTestText.Text, TempString);

     // copy the output back into the edit box
     edtTestText.Text := TempString;
end; {TfrmRSATest.btnEncryptClick}

{*******************************************************************************
* Procedure : TfrmRSATest.btnDecryptClick                                      *
********************************************************************************
* Purpose   : Calls the TRSA procedure for decrypting data using the currently *
*             loaded keys                                                      *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.btnDecryptClick(Sender: TObject);
var
     TempString: string;
begin
     // decrypt the string in the edit box
     RSA1.DecryptString(edtTestText.Text, TempString);

     // copy the output back into the edit box
     edtTestText.Text := TempString;
end; {TfrmRSATest.btnDecryptClick}

{*******************************************************************************
* Procedure : TfrmRSATest.RSA1StatusUpdate                                     *
********************************************************************************
* Purpose   : Passes the status messages fromTRSA to the status bar on the     *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.RSA1StatusUpdate(Sender: TObject);
begin
     sbStatus.SimpleText := RSA1.Status;
end; {RSA1StatusUpdate}

{*******************************************************************************
* Procedure : TfrmRSATest.rbn0512Click                                         *
********************************************************************************
* Purpose   : Sets the current key length of TRSA to 512 bits                  *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.rbn512Click(Sender: TObject);
begin
     RSA1.KeyBits := kb0512;
end; {TfrmRSATest.rbn512Click}

{*******************************************************************************
* Procedure : TfrmRSATest.rbn1024Click                                         *
********************************************************************************
* Purpose   : Sets the current key length of TRSA to 1024 bits                 *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.rbn1024Click(Sender: TObject);
begin
     RSA1.KeyBits := kb1024;
end; {TfrmRSATest.rbn1024Click}

{*******************************************************************************
* Procedure : TfrmRSATest.rbn2048Click                                         *
********************************************************************************
* Purpose   : Sets the current key length of TRSA to 2048 bits                 *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.rbn2048Click(Sender: TObject);
begin
     RSA1.KeyBits := kb2048;
end; {TfrmRSATest.rbn2048Click}

{*******************************************************************************
* Procedure : TfrmRSATest.btnPublicKeySetClick                                 *
********************************************************************************
* Purpose   : Sets the public key to the value held in the edit box            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.btnPublicKeySetClick(Sender: TObject);
begin
     RSA1.PublicKey := edtPublicKey.Text;
end; {TfrmRSATest.btnPublicKeySetClick}

{*******************************************************************************
* Procedure : TfrmRSATest.btnPrivateKeySetClick                                *
********************************************************************************
* Purpose   : Sets the private key to the value held in the edit box           *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.btnPrivateKeySetClick(Sender: TObject);
begin
     RSA1.PrivateKey := edtPrivateKey.Text;
end; {TfrmRSATest.btnPrivateKeySetClick}

{*******************************************************************************
* Procedure : TfrmRSATest.btnSignClick                                         *
********************************************************************************
* Purpose   : Creates a TRSA signature                                         *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.btnSignClick(Sender: TObject);
var
     Digest: TDigest;
     i: integer;
begin
     SHA1.Init;
     for i := 1 to Length(edtTestText.Text) do
     begin
          SHA1.HashByte(Ord(edtTestText.Text[i]));
     end; {for i}
     Digest := SHA1.Finish;
     RSA1.Sign(Digest, 20);
     edtSignature.Text := RSA1.Signature;
end; {TfrmRSATest.btnSignClick}

{*******************************************************************************
* Procedure : TfrmRSATest.btnVerifyClick                                       *
********************************************************************************
* Purpose   : Verifies a TRSA signature                                        *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.btnVerifyClick(Sender: TObject);
var
     Digest: TDigest;
     i: integer;
     Verified: Boolean;
begin
     // create the hash of the text
     SHA1.Init;

     for i := 1 to Length(edtTestText.Text) do
     begin
          SHA1.HashByte(Ord(edtTestText.Text[i]));
     end; {for i}

     // get the digest in an array
     Digest := SHA1.Finish;

     // verify that the sig string in RSA1.Signature matches the digest
     Verified := RSA1.Verify(Digest, 20);

     if Verified then
     begin
          ShowMessage('Signature verified OK.');
     end
     else
     begin
          ShowMessage('Signature did not verify.');
     end;
end; {TfrmRSATest.btnVerifyClick}

{*******************************************************************************
* Procedure : TfrmRSATest.btnSignatureSetClick                                 *
********************************************************************************
* Purpose   : Sets the signature to the value held in the edit box             *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.btnSignatureSetClick(Sender: TObject);
begin
     RSA1.Signature := edtSignature.Text;
end; {TfrmRSATest.btnSignatureSetClick}

{*******************************************************************************
* Procedure : TfrmRSATest.btnSymKeyClick                                       *
********************************************************************************
* Purpose   : Tests that the symmetric key encryption works                    *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.btnSymKeyClick(Sender: TObject);
var
     SymKey: TSymKey;
     EncSymKey: String;
     TestKey: TSymKey;
     i: integer;
begin
     // load a fictional symmetric key
     for i := 0 to 7 do
     begin
          SymKey[i] := i;
     end;

     // set the key length to 64 bits (8 bytes)
     RSA1.SymKeyLength := sk64;

     // encrypt the key
     EncSymKey := RSA1.EncryptKey(SymKey);

     // decrypt the key
     RSA1.DecryptKey(EncSymKey, TestKey);

     // test that the encryption & decryption went ok
     for i := 0 to 7 do
     begin
          if SymKey[i] <> i then
          begin
               ShowMessage('Error in symmetric key processing');
          end; {if}
     end; {for}

     ShowMessage ('Symmetric key processing ok!');
end; {TfrmRSATest.btnSymKeyClick}

{*******************************************************************************
* Procedure : TfrmRSATest.FormCreate                                           *
********************************************************************************
* Purpose   : Displays the RSA version number                                  *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.FormCreate(Sender: TObject);
begin
     // retrieve the version of the RSA component
     edtVersion.Text := RSA1.GetVersion;
end; {TfrmRSATest.FormCreate}

{*******************************************************************************
* Procedure : TfrmRSATest.btnPubKeyLoadClick                                   *
********************************************************************************
* Purpose   : Loads the public key quickly with a precomputed key              *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.btnPubKeyLoadClick(Sender: TObject);
begin
     case RSA1.KeyBits of
          kb0512:
               edtPublicKey.Text := '++11Ik:Rel4qH+uPYrjJjeVbxJhk1GdkOMgYUND9FBvRRymMwggm4GII42ScGEt4+UqEDMrbD3kVsLK0kjdSC2oTdABjU';
          kb1024:
               edtPublicKey.Text := '++11Ik:m08k-MA3AiLsWQgjs-XvDm7Tv5ng+g8il8rEdE1LSHPS16WyOYCqyYcPVGxBB0M04YPJofXHi8loLZRpJQ+NMqbY+RP9jJGTbL9u+rdjmpYXyHSBpIDGlyb73E+wIlvnxy-qB2iBG5ToBqH2rCd+P2xMOx2IrA1hMOh9OQV5LPs';
          kb2048:
          begin
               edtPublicKey.Text := '++11Ik:Ns51TTRolJrtj5L9GuigB-STcrk8ILxSZ2SvuxMuwVa7bnTBhRjbAh7DBnrS7idtR8IdHouHXOQ0TJdNsftWVVZVa8bscUwFgvSZUOlhyvRuj2zPEPUFUM+ktABxHhdTXgLMq3Qk0A7Kc0MKkcXpvwdk8+ydnDk2vkAiaUfELGJclJ83tpqzQXmzM';
               edtPublicKey.Text := edtPublicKey.Text + '9F+PkerwazTaeHCxVEI64TaAEOMoK0TG6khaIgg7qMmGCUwIIeL73RhoDaiQODofupA3pYnaXj1lgbQwg7Lwq0rf9w8RGM1IUfaMudOScHAE8YrIr6B1BLLciZokqs4sHQln7pQ529VTy3EwKNX3s8ZRLpDfk';
          end;
     end; {case}
     RSA1.PublicKey := edtPublicKey.Text;
end; {TfrmRSATest.btnPubKeyLoadClick}

{*******************************************************************************
* Procedure : TfrmRSATest.btnPubKeyLoadClick                                   *
********************************************************************************
* Purpose   : Loads the private key quickly with a precomputed key             *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmRSATest.btnPrivKeyLoadClick(Sender: TObject);
begin
     case RSA1.KeyBits of
          kb0512:
               edtPrivateKey.Text := 'Rel4qH+uPYrjJjeVbxJhk1GdkOMgYUND9FBvRRymMwggm4GII42ScGEt4+UqEDMrbD3kVsLK0kjdSC2oTdABjU:8jc39wxL0gG9oYj4yVEGWEkDJ1O+DaLFHppxwRKy1PdcPNG7Owv5dzjW2mX6tdAX-82Xx0HhMhBsVMPrR7w5mU';
          kb1024:
          begin
               edtPrivateKey.Text := 'm08k-MA3AiLsWQgjs-XvDm7Tv5ng+g8il8rEdE1LSHPS16WyOYCqyYcPVGxBB0M04YPJofXHi8loLZRpJQ+NMqbY+RP9jJGTbL9u+rdjmpYXyHSBpIDGlyb73E+wIlvnxy-qB2iBG5ToBqH2rCd+P2xMOx2IrA1hMOh9OQV5LPs';
               edtPrivateKey.Text := edtPrivateKey.Text + ':-UpSDvQafjxM+W4cREvvCwdm4LoNkvdi9xg-3HoviUlsq4VIvEXixFmAs0rf3qq0foVoB7RxV0LJ5hTGfLR-OuhKV7ZRTN1YEzj4aKajlNzcX4fkcf3Y5u-R0D3v7xsaWYhuH5ZOtIwMImkWLQbCgIcJSwM542P2RCRAC1hdE1o';
          end;
          kb2048:
          begin
               edtPrivateKey.Text := 'Ns51TTRolJrtj5L9GuigB-STcrk8ILxSZ2SvuxMuwVa7bnTBhRjbAh7DBnrS7idtR8IdHouHXOQ0TJdNsftWVVZVa8bscUwFgvSZUOlhyvRuj2zPEPUFUM+ktABxHhdTXgLMq3Qk0A7Kc0MKkcXpvwdk8+ydnDk2vkAiaUfELGJclJ83tpqzQXmzM';
               edtPrivateKey.Text := edtPrivateKey.Text + '9F+PkerwazTaeHCxVEI64TaAEOMoK0TG6khaIgg7qMmGCUwIIeL73RhoDaiQODofupA3pYnaXj1lgbQwg7Lwq0rf9w8RGM1IUfaMudOScHAE8YrIr6B1BLLciZokqs4sHQln7pQ529VTy3EwKNX3s8ZRLpDfk';
               edtPrivateKey.Text := edtPrivateKey.Text + ':9EeqZs5KCYOFlKQt5wWMQwo4EZnGUzLa5LQIc3qSKekyxzVsvoDIYm02dQsqVON3tHwEt-BVrkONg9E-sOMI34VPNL8J7H6ViyybGw78DEPPPzHrFEYz+JAjAQVnwKgYSDxS4Q87YPi5yBVg9nikIQNkHnLmeuL8+XxlRPuqIQ9IUm7JUMlwHzzYAPCST';
               edtPrivateKey.Text := edtPrivateKey.Text + 'K6O31yC4VFDqjvn7pzO91MT7LRxjvnXEE7kfBMjb+5T4p-i7m-NyvywsK-kha6R9ToLBAbboarBkAjwYqUMQRE8OYvY8BVrISKQLpYLGDHx5BzZp3s72PqWKGQWyHjl3FjwF5sB9GmgiCGEeXWtI81FzE';
          end;
     end; {case}
     RSA1.PrivateKey := edtPrivateKey.Text;
end; {TfrmRSATest.btnPrivKeyLoadClick}

end.
